<?php

namespace Goldenscarab\Modulus\Helpers;

class Dynamik
{

    private $source;

    public function __construct($source)
    {
        $this->source = $source;
    }

    /**
     * Création d'une valeur depuis une condition
     *
     * @param array $data
     * $structure =
     * [
     *      'value'      => ':clef.de.recherche',
     *      'operator'   => '==',
     *      'comparator' => 'valeur de comparaison',
     *      'true'       => 'valeur si vrai',
     *      'false'      => 'valeur si faux'
     * ]
     * @return Mixte La valeur
     */
    public function getConditionalValue(array $data)
    {
        $val        = $this->getValue($data['value']);
        $operator   = data_get($data, 'operator', '==');
        $true       = data_get($data, 'true');
        $comparator = data_get($data, 'comparator', true);
        $false      = data_get($data, 'false');

        switch ($operator) {
            case '==' :
                $value = $val == $comparator ? $true : $false;
                break;
            case '===' :
                $value = $val === $comparator ? $true : $false;
                break;
            case '>' :
                $value = $val > $comparator ? $true : $false;
                break;
            case '<' :
                $value = $val < $comparator ? $true : $false;
                break;
            case '!=' :
                $value = $val != $comparator ? $true : $false;
                break;
            case '!==' :
                $value = $val !== $comparator ? $true : $false;
                break;
            case '<=' :
                $value = $val <= $comparator ? $true : $false;
                break;
            case '>=' :
                $value = $val >= $comparator ? $true : $false;
                break;
            default :
                $value = $false;
        }

        return $this->getValue($value);
    }

    /**
     * Création d'une valeur depuis un template
     *
     * @param array $data
     * $structure =
     * [
     *      'format' => 'le template à utiliser',
     *      'args'   => [':clef.de.recherche', 'Valeur fixe'],
     * ]
     * @return Mixte La valeur
     */
    public function getFormatedValue(array $data)
    {
        $value  = null;
        $format = $data['format'];
        $args   = $data['args'];

        // Traitement des % seuls pour éviter les conflits
        $format = preg_replace('/(%)[^0-9sd.\']/i', '%%"', $format);

        // Préparation des arguments pour le template
        $arguments = $this->getValue($args);

        if (!is_array($arguments)) {
            $arguments = [$arguments];
        }

        $value = vsprintf($format, $arguments);

        return $value;
    }

    /**
     * Récupération d'une valeur depuis une fonction callback
     * $structure =
     * [
     *      'function' => 'function_callback',
     *      'args'      => [':clef.de.recherche', 'Valeur fixe'],
     * ]
     * @param array $data
     * @return Mixte
     */
    public function getCallbackValue($data)
    {
        $value    = null;
        $function = $data['function'];
        $args     = data_get($data, 'args');

        // Traitement sur le nom de la fonction (suppression des parenthèses)
        $function = preg_replace('/(\(.*)/', '', $function);

        // Préparation des arguments pour la fonction
        $arguments = $this->getValue($args);

        if (!is_array($arguments)) {
            $arguments = [$arguments];
        }

        $value = call_user_func_array($function, $arguments);

        return $value;
    }

    /**
     * Récupération d'une valeur url depuis une route avec des arguments
     * $structure =
     * [
     *      'route' => 'ma.route',
     *      'args'  => [':id', 'param' => ':name'],
     * ]
     *
     * @param array $data
     * @return String
     */
    public function getUrlValue($data)
    {
        $value = null;

        if (is_array($data)) {
            if (array_key_exists('route', $data)) {
                $route = $this->getValue($data['route']);
                $args = $data['args'];

                // Préparation des arguments pour la route
                if (!is_array($args)) {
                    $args = [$args];
                }
                $arguments = [];
                foreach ($args as $key => $val) {
                    $arguments[$key] = $this->getValue($val);
                }

                $value = route($route, $arguments);
            } else {
                $value = $this->getValue($data);
            }

        } else {
            $value = $data;
        }

        return $value;
    }

    /**
     * Récupération d'une valeur dynamique (en préfixant avec « : »)
     * Exemple :
     *  :name
     *  :updated_at->diffForHumans()
     *  [ 'template'  => [...]]
     *  [ 'condition' => [...]]
     *  [ 'callback'  => [...]]
     *  date('Y')
     *  'Ma valeur fixe'
     *
     * @param [type] $data
     * @return void
     */
    public function getValue($data)
    {
        $value = null;

        if (is_array($data)) {
            $values = [];

            // Parcours des arguments
            foreach ($data as $key => $d) {

                if ($key === 'condition') {
                    $values[] = $this->getConditionalValue($d);
                } else if ($key === 'template') {
                    $values[] = $this->getFormatedValue($d);
                } else if ($key === 'callback') {
                    $values[] = $this->getCallbackValue($d);
                } else if ($key === 'url') {
                    $values[] = $this->getUrlValue($d);
                } else {
                    $values[] = $this->getValue($d);
                }
            }

            // Si une seule valeur alors on supprime le tableau
            if (count($values) == 1) {
                $value = $values[0];
            } else {
                $value = $values;
            }

        } else {
            // Valeur dynamique
            if (preg_match('/^:.*$/', $data)) {
                $data = trim($data, ':');
                $value = mixte_get($this->source, $data);
            } else {
                $value = $data;
            }
        }

        return $value;
    }

    /**
     * Remplacement des données dynamik d'un tableau d'argument en valeur
     *
     * @param array $datas
     * @return array
     */
    public function dynamikize(array $datas): array
    {
        $dynamikized = [];
        foreach ($datas as $key => $data) {

            if ($this->isDynamik($data)) {
                $value = $this->getValue($data);
            } else {
                $value = $data;
            }

            if (is_array($value)) {
                $dynamik = $this->dynamikize($value);
            } else {
                $dynamik = $value;
            }

            $dynamikized[$key] = $dynamik;
        }

        return $dynamikized;
    }

    /**
     * Test si une valeur est de type dynamik
     *
     * @param Mixte $data
     * @return boolean
     */
    private function isDynamik($data): bool
    {
        $keys_dynamik = ['condition', 'template', 'callback', 'url'];

        if (is_array($data)) {
            foreach ($keys_dynamik as $key) {
                if (array_key_exists($key, $data)) {
                    return true;
                }
            }
        } else if (preg_match('/^:.*$/', $data)) {
            return true;
        }

        return false;
    }
}
